from mpyzeebe.grpc_internals.zeebe_job_adapter import ZeebeJobAdapter
from mpyzeebe.grpc_internals.zeebe_message_adapter import ZeebeMessageAdapter
from mpyzeebe.grpc_internals.zeebe_workflow_adapter import ZeebeWorkflowAdapter


# Mixin class
class ZeebeAdapter(ZeebeWorkflowAdapter, ZeebeJobAdapter, ZeebeMessageAdapter):
    pass



__version__ = "2.3.1"

from mpyzeebe import exceptions
from mpyzeebe.client.client import ZeebeClient
from mpyzeebe.credentials.camunda_cloud_credentials import CamundaCloudCredentials
from mpyzeebe.credentials.oauth_credentials import OAuthCredentials
from mpyzeebe.job.job import Job
from mpyzeebe.job.job_status import JobStatus
from mpyzeebe.task.exception_handler import ExceptionHandler
from mpyzeebe.task.task_decorator import TaskDecorator
from mpyzeebe.worker.task_router import ZeebeTaskRouter
from mpyzeebe.worker.worker import ZeebeWorker



from typing import List

from mpyzeebe.task.task_decorator import TaskDecorator


class ZeebeDecoratorBase(object):
    def __init__(self, before: List[TaskDecorator] = None, after: List[TaskDecorator] = None):
        self._before: List[TaskDecorator] = before or []
        self._after: List[TaskDecorator] = after or []

    def before(self, *decorators: TaskDecorator) -> None:
        self._before.extend(decorators)

    def after(self, *decorators: TaskDecorator) -> None:
        self._after.extend(decorators)



import logging
import os
import time
import grpc
from zeebe_grpc.gateway_pb2_grpc import GatewayStub

from mpyzeebe.credentials.base_credentials import BaseCredentials
from mpyzeebe.exceptions import ZeebeBackPressure, ZeebeGatewayUnavailable, ZeebeInternalError

logger = logging.getLogger(__name__)


class ZeebeAdapterBase(object):
    def __init__(self, hostname: str = None, port: int = None, credentials: BaseCredentials = None,
                 channel: grpc.Channel = None, secure_connection: bool = False, max_connection_retries: int = -1):
        if channel:
            self.connection_uri = None
            self._channel = channel
        else:
            self.connection_uri = self._get_connection_uri(hostname, port, credentials)
            self._channel = self._create_channel(self.connection_uri, credentials, secure_connection)

        self.secure_connection = secure_connection
        self.connected = False
        self.retrying_connection = True
        self._channel.subscribe(self._check_connectivity, try_to_connect=True)
        self._gateway_stub = GatewayStub(self._channel)
        self._max_connection_retries = max_connection_retries
        self._current_connection_retries = 0

    @staticmethod
    def _get_connection_uri(hostname: str = None, port: int = None, credentials: BaseCredentials = None) -> str:
        if credentials and credentials.get_connection_uri():
            return credentials.get_connection_uri()
        if hostname or port:
            return f"{hostname or 'localhost'}:{port or 26500}"
        else:
            return os.getenv("ZEEBE_ADDRESS", "localhost:26500")

    @staticmethod
    def _create_channel(connection_uri: str, credentials: BaseCredentials = None,
                        secure_connection: bool = False) -> grpc.Channel:
        maxMsgLength = 30 * 1024 * 1024
        if credentials:
            return grpc.secure_channel(connection_uri,credentials.grpc_credentials, options=[('grpc.max_message_length', maxMsgLength),
                                         ('grpc.max_send_message_length', maxMsgLength),
                                         ('grpc.max_receive_message_length', maxMsgLength)])
        elif secure_connection:
            return grpc.secure_channel(connection_uri, grpc.ssl_channel_credentials(),options=[('grpc.max_message_length', maxMsgLength),
                                         ('grpc.max_send_message_length', maxMsgLength),
                                         ('grpc.max_receive_message_length', maxMsgLength)])
        else:
            return grpc.insecure_channel(connection_uri,options=[('grpc.max_message_length', maxMsgLength),
                                         ('grpc.max_send_message_length', maxMsgLength),
                                         ('grpc.max_receive_message_length', maxMsgLength)])


    def _check_connectivity(self, value: grpc.ChannelConnectivity) -> None:
        logger.debug(f"Grpc channel connectivity changed to: {value}")
        self.connected = False

        if value in [grpc.ChannelConnectivity.READY, grpc.ChannelConnectivity.IDLE]:
            logger.debug(f"Connected to {self.connection_uri or 'zeebe'}")
            self.connected = True
            self.retrying_connection = False
            self._current_connection_retries = 0

        elif value == grpc.ChannelConnectivity.CONNECTING:
            logger.debug(f"Connecting to {self.connection_uri or 'zeebe'}.")
            self.retrying_connection = True

        elif value == grpc.ChannelConnectivity.TRANSIENT_FAILURE:
            if self._should_retry():
                logger.warning(f"Lost connection to {self.connection_uri or 'zeebe'}. Retrying...")
                time.sleep(10)
                if self._current_connection_retries > self._max_connection_retries/2:
                    time.sleep(20)
                self.retrying_connection = True
                self._current_connection_retries = self._current_connection_retries + 1
            else:
                logger.error(f"Failed to establish connection to {self.connection_uri or 'zeebe'}. Not recoverable")
                self._close()
                self.retrying_connection = False
                raise ConnectionAbortedError(f"Lost connection to {self.connection_uri or 'zeebe'}")

        elif value == grpc.ChannelConnectivity.SHUTDOWN:
            logger.warning(f"Shutting down grpc channel to {self.connection_uri or 'zeebe'}")
            self.retrying_connection = False

    def _should_retry(self):
        self._max_connection_retries = -1
        return self._max_connection_retries == -1 or self._current_connection_retries < self._max_connection_retries

    def _common_zeebe_grpc_errors(self, rpc_error: grpc.RpcError):
        if self.is_error_status(rpc_error, grpc.StatusCode.RESOURCE_EXHAUSTED):
            if not "RESOURCE_EXHAUSTED" in str(rpc_error):
                logger.error(rpc_error)
            pass
           # raise ZeebeBackPressure()
        elif self.is_error_status(rpc_error, grpc.StatusCode.UNAVAILABLE):
            self._current_connection_retries += 1
            if not self._should_retry():
                self._close()
            raise ZeebeGatewayUnavailable()
        elif self.is_error_status(rpc_error, grpc.StatusCode.INTERNAL):
            self._current_connection_retries += 1
            if not self._should_retry():
                self._close()
            raise ZeebeInternalError()
        else:
            logger.error(rpc_error)
            pass
          #  raise rpc_error

    @staticmethod
    def is_error_status(rpc_error: grpc.RpcError, status_code: grpc.StatusCode):
        return rpc_error._state.code == status_code

    def _close(self):
        try:
            self._channel.close()
        except Exception as e:
            logger.exception(f"Failed to close channel, {type(e).__name__} exception was raised")



from mpyzeebe.credentials.oauth_credentials import OAuthCredentials
from mpyzeebe.exceptions import InvalidOAuthCredentials, InvalidCamundaCloudCredentials


class CamundaCloudCredentials(OAuthCredentials):
    def __init__(self, client_id: str, client_secret: str, cluster_id: str):
        try:
            super().__init__(url="https://login.cloud.camunda.io/oauth/token", client_id=client_id,
                             client_secret=client_secret, audience=f"{cluster_id}.zeebe.camunda.io")
        except InvalidOAuthCredentials:
            raise InvalidCamundaCloudCredentials(client_id=client_id, cluster_id=cluster_id)

    def get_connection_uri(self) -> str:
        return f"{self.audience}:443"



from typing import List, Callable, Dict

from mpyzeebe.task.exception_handler import ExceptionHandler
from mpyzeebe.task.task import Task
from mpyzeebe.task.task_decorator import TaskDecorator
from mpyzeebe.worker.task_handler import ZeebeTaskHandler, default_exception_handler


class ZeebeTaskRouter(ZeebeTaskHandler):
    def _dict_task(self, task_type: str, exception_handler: ExceptionHandler = default_exception_handler,
                   timeout: int = 10000, max_jobs_to_activate: int = 32, before: List[TaskDecorator] = None,
                   after: List[TaskDecorator] = None, variables_to_fetch: List[str] = None):
        def wrapper(fn: Callable[..., Dict]):
            nonlocal variables_to_fetch
            if not variables_to_fetch:
                variables_to_fetch = self._get_parameters_from_function(fn)

            task = self._create_task(task_type=task_type, task_handler=fn, exception_handler=exception_handler,
                                     timeout=timeout, max_jobs_to_activate=max_jobs_to_activate, before=before,
                                     after=after, variables_to_fetch=variables_to_fetch)

            self.tasks.append(task)
            return fn

        return wrapper

    def _non_dict_task(self, task_type: str, variable_name: str,
                       exception_handler: ExceptionHandler = default_exception_handler, timeout: int = 10000,
                       max_jobs_to_activate: int = 32, before: List[TaskDecorator] = None,
                       after: List[TaskDecorator] = None, variables_to_fetch: List[str] = None):
        def wrapper(fn: Callable[..., Dict]):
            nonlocal variables_to_fetch
            if not variables_to_fetch:
                variables_to_fetch = self._get_parameters_from_function(fn)

            dict_fn = self._single_value_function_to_dict(variable_name=variable_name, fn=fn)

            task = self._create_task(task_type=task_type, task_handler=dict_fn, exception_handler=exception_handler,
                                     timeout=timeout, max_jobs_to_activate=max_jobs_to_activate, before=before,
                                     after=after, variables_to_fetch=variables_to_fetch)

            self.tasks.append(task)
            return fn

        return wrapper

    def _create_task(self, task_type: str, task_handler: Callable, exception_handler: ExceptionHandler,
                     timeout: int = 10000, max_jobs_to_activate: int = 32, before: List[TaskDecorator] = None,
                     after: List[TaskDecorator] = None, variables_to_fetch: List[str] = None) -> Task:
        task = Task(task_type=task_type, task_handler=task_handler, exception_handler=exception_handler,
                    timeout=timeout, max_jobs_to_activate=max_jobs_to_activate, variables_to_fetch=variables_to_fetch)
        return self._add_decorators_to_task(task, before or [], after or [])

    def _add_decorators_to_task(self, task: Task, before: List[TaskDecorator],
                                after: List[TaskDecorator]) -> Task:
        before_decorators = self._before.copy()
        before_decorators.extend(before)

        after.extend(self._after)

        task.before(*before_decorators)
        task.after(*after)
        return task



import json
import os
from typing import Dict

import grpc
from zeebe_grpc.gateway_pb2 import CreateWorkflowInstanceRequest, CreateWorkflowInstanceWithResultRequest, \
    CancelWorkflowInstanceRequest, WorkflowRequestObject, DeployWorkflowRequest, DeployWorkflowResponse

from mpyzeebe.exceptions import InvalidJSON, WorkflowNotFound, WorkflowInstanceNotFound, WorkflowHasNoStartEvent, \
    WorkflowInvalid
from mpyzeebe.grpc_internals.zeebe_adapter_base import ZeebeAdapterBase


class ZeebeWorkflowAdapter(ZeebeAdapterBase):
    def create_workflow_instance(self, bpmn_process_id: str, version: int, variables: Dict) -> int:
        try:
            response = self._gateway_stub.CreateWorkflowInstance(
                CreateWorkflowInstanceRequest(bpmnProcessId=bpmn_process_id, version=version,
                                              variables=json.dumps(variables)))
            return response.workflowInstanceKey
        except grpc.RpcError as rpc_error:
            self._create_workflow_errors(rpc_error, bpmn_process_id, version, variables)

    def create_workflow_instance_with_result(self, bpmn_process_id: str, version: int, variables: Dict,
                                             timeout: int, variables_to_fetch) -> Dict:
        try:
            response = self._gateway_stub.CreateWorkflowInstanceWithResult(
                CreateWorkflowInstanceWithResultRequest(
                    request=CreateWorkflowInstanceRequest(bpmnProcessId=bpmn_process_id, version=version,
                                                          variables=json.dumps(variables)),
                    requestTimeout=timeout, fetchVariables=variables_to_fetch))
            return json.loads(response.variables)
        except grpc.RpcError as rpc_error:
            self._create_workflow_errors(rpc_error, bpmn_process_id, version, variables)

    def _create_workflow_errors(self, rpc_error: grpc.RpcError, bpmn_process_id: str, version: int,
                                variables: Dict) -> None:
        if self.is_error_status(rpc_error, grpc.StatusCode.NOT_FOUND):
            raise WorkflowNotFound(bpmn_process_id=bpmn_process_id, version=version)
        elif self.is_error_status(rpc_error, grpc.StatusCode.INVALID_ARGUMENT):
            raise InvalidJSON(
                f"Cannot start workflow: {bpmn_process_id} with version {version}. Variables: {variables}")
        elif self.is_error_status(rpc_error, grpc.StatusCode.FAILED_PRECONDITION):
            raise WorkflowHasNoStartEvent(bpmn_process_id=bpmn_process_id)
        else:
            self._common_zeebe_grpc_errors(rpc_error)

    def cancel_workflow_instance(self, workflow_instance_key: int) -> None:
        try:
            self._gateway_stub.CancelWorkflowInstance(
                CancelWorkflowInstanceRequest(workflowInstanceKey=workflow_instance_key))
        except grpc.RpcError as rpc_error:
            if self.is_error_status(rpc_error, grpc.StatusCode.NOT_FOUND):
                raise WorkflowInstanceNotFound(workflow_instance_key=workflow_instance_key)
            else:
                self._common_zeebe_grpc_errors(rpc_error)

    def deploy_workflow(self, *workflow_file_path: str) -> DeployWorkflowResponse:
        try:
            return self._gateway_stub.DeployWorkflow(
                DeployWorkflowRequest(workflows=map(self._get_workflow_request_object, workflow_file_path)))
        except grpc.RpcError as rpc_error:
            if self.is_error_status(rpc_error, grpc.StatusCode.INVALID_ARGUMENT):
                raise WorkflowInvalid()
            else:
                self._common_zeebe_grpc_errors(rpc_error)

    @staticmethod
    def _get_workflow_request_object(workflow_file_path: str) -> WorkflowRequestObject:
        with open(workflow_file_path, "rb") as file:
            return WorkflowRequestObject(name=os.path.split(workflow_file_path)[-1],
                                         definition=file.read())



import logging
from abc import abstractmethod
from typing import Tuple, List, Callable, Dict

from mpyzeebe.decorators.zeebe_decorator_base import ZeebeDecoratorBase
from mpyzeebe.exceptions import NoVariableNameGiven, TaskNotFound, DuplicateTaskType
from mpyzeebe.job.job import Job
from mpyzeebe.task.exception_handler import ExceptionHandler
from mpyzeebe.task.task import Task
from mpyzeebe.task.task_decorator import TaskDecorator

logger = logging.getLogger(__name__)


def default_exception_handler(e: Exception, job: Job) -> None:
    logger.warning(f"Task type: {job.type} - failed job {job}. Error: {e}.")
    job.set_failure_status(f"Failed job. Error: {e}")


class ZeebeTaskHandler(ZeebeDecoratorBase):
    def __init__(self, before: List[TaskDecorator] = None, after: List[TaskDecorator] = None):
        """
        Args:
            before (List[TaskDecorator]): Decorators to be performed before each task
            after (List[TaskDecorator]): Decorators to be performed after each task
        """
        super().__init__(before, after)
        self.tasks: List[Task] = []

    def task(self, task_type: str, exception_handler: ExceptionHandler = default_exception_handler,
             variables_to_fetch: List[str] = None, timeout: int = 10000, max_jobs_to_activate: int = 32,
             before: List[TaskDecorator] = None, after: List[TaskDecorator] = None, single_value: bool = False,
             variable_name: str = None):
        """
        Decorator to create a task

        Args:
            before (List[TaskDecorator]): All decorators which should be performed before the task.
            after (List[TaskDecorator]): All decorators which should be performed after the task.
            timeout (int): How long Zeebe should wait before the job is retried. Default: 10000 milliseconds
            single_value (bool): If the function returns a single value (int, string, list) and not a dictionary set
                                 this to True. Default: False
            variable_name (str): If single_value then this will be the variable name given to zeebe:
                                        { <variable_name>: <function_return_value> }
            timeout (int): Maximum duration of the task in milliseconds. If the timeout is surpasses Zeebe will give up
                            on the job and retry it. Default: 10000
            max_jobs_to_activate (int):  Maximum jobs the worker will execute in parallel (of this task). Default: 32

        Raises:
            DuplicateTaskType: If a task from the router already exists in the worker

        """
        self._is_task_duplicate(task_type)

        if single_value and not variable_name:
            raise NoVariableNameGiven(task_type=task_type)

        elif single_value and variable_name:
            return self._non_dict_task(task_type=task_type, variable_name=variable_name, timeout=timeout,
                                       max_jobs_to_activate=max_jobs_to_activate, exception_handler=exception_handler,
                                       before=before, after=after, variables_to_fetch=variables_to_fetch)

        else:
            return self._dict_task(task_type=task_type, exception_handler=exception_handler, before=before, after=after,
                                   timeout=timeout, max_jobs_to_activate=max_jobs_to_activate,
                                   variables_to_fetch=variables_to_fetch)

    @abstractmethod
    def _dict_task(self, task_type: str, exception_handler: ExceptionHandler = default_exception_handler,
                   timeout: int = 10000, max_jobs_to_activate: int = 32, before: List[TaskDecorator] = None,
                   after: List[TaskDecorator] = None, variables_to_fetch: List[str] = None):
        raise NotImplemented()

    @abstractmethod
    def _non_dict_task(self, task_type: str, variable_name: str,
                       exception_handler: ExceptionHandler = default_exception_handler, timeout: int = 10000,
                       max_jobs_to_activate: int = 32, before: List[TaskDecorator] = None,
                       after: List[TaskDecorator] = None, variables_to_fetch: List[str] = None):
        raise NotImplemented()

    @staticmethod
    def _single_value_function_to_dict(variable_name: str, fn: Callable) -> Callable[..., Dict]:
        def inner_fn(*args, **kwargs):
            return {variable_name: fn(*args, **kwargs)}

        return inner_fn

    @staticmethod
    def _get_parameters_from_function(fn: Callable) -> List[str]:
        parameters = fn.__code__.co_varnames
        if "args" in parameters:
            return []
        elif "kwargs" in parameters:
            return []
        else:
            return list(parameters)

    def _is_task_duplicate(self, task_type: str) -> None:
        try:
            self.get_task(task_type)
            raise DuplicateTaskType(task_type)
        except TaskNotFound:
            return

    def remove_task(self, task_type: str) -> Task:
        """
        Remove a task

        Args:
            task_type (str): The type of the wanted task

        Returns:
            Task: The task that was removed

        Raises:
             TaskNotFound: If no task with specified type exists

        """
        task_index = self._get_task_index(task_type)
        return self.tasks.pop(task_index)

    def get_task(self, task_type: str) -> Task:
        """
        Get a task by its type

        Args:
            task_type (str): The type of the wanted task

        Returns:
            Task: The wanted task

        Raises:
             TaskNotFound: If no task with specified type exists

        """
        return self._get_task_and_index(task_type)[0]

    def _get_task_index(self, task_type: str) -> int:
        return self._get_task_and_index(task_type)[-1]

    def _get_task_and_index(self, task_type: str) -> Tuple[Task, int]:
        for index, task in enumerate(self.tasks):
            if task.type == task_type:
                return task, index
        raise TaskNotFound(f"Could not find task {task_type}")



#!/usr/bin/env python
# -*- coding:utf-8 -*-


from setuptools import setup, find_packages
from setuptools.command.install import install
from subprocess import call

setup(
    name = 'mpyzeebe',
    version = '0.0.6',
    keywords='wx',
    description = 'a library for nmap scan',
    license = 'MIT License',
    url = 'https://192.168.1.146:8081/repo/packages',
    author = 'superman',
    author_email = '646390966@qq.com',
    packages = find_packages(),
    include_package_data = True,
    platforms = 'any',
    install_requires = [
"oauthlib==3.1.0", "requests-oauthlib==1.3.0", "zeebe-grpc==0.26.0.0"],
)

